@@ -1,22 +1,67 @@
 import collections
 import csv
-import re
 import os
-from os import walk
-from os.path import splitext, join, dirname
+import re
 import tempfile
-from ovos_utils.bracket_expansion import expand_options
-from ovos_utils.log import LOG
-from ovos_utils.system import search_mycroft_core_location
-import time
-from os.path import dirname
+from os import walk
+from os.path import dirname, splitext, join
+from sys import platform
+from threading import RLock
+from typing import Optional, List
 
-from watchdog.observers import Observer
 from watchdog.events import FileSystemEventHandler
+from watchdog.observers import Observer
+
+from ovos_utils.bracket_expansion import expand_template
+from ovos_utils.log import LOG, log_deprecation
 
 
-def get_temp_path(*args):
-    """Generate a valid path in the system temp directory.
+def ensure_directory_exists(directory, domain=None):
+    """ Create a directory and give access rights to all
+
+    Args:
+        domain (str): The IPC domain.  Basically a subdirectory to prevent
+            overlapping signal filenames.
+
+    Returns:
+        str: a path to the directory
+    """
+    if domain:
+        directory = os.path.join(directory, domain)
+
+    # Expand and normalize the path
+    directory = os.path.normpath(directory)
+    directory = os.path.expanduser(directory)
+
+    if not os.path.isdir(directory):
+        try:
+            save = os.umask(0)
+            os.makedirs(directory, 0o777)  # give everyone rights to r/w here
+        except OSError:
+            LOG.warning("Failed to create: " + directory)
+            pass
+        finally:
+            os.umask(save)
+
+    return directory
+
+
+def to_alnum(skill_id: str) -> str:
+    """
+    Convert a skill id to only alphanumeric characters
+     Non-alphanumeric characters are converted to "_"
+
+    Args:
+        skill_id (str): identifier to be converted
+    Returns:
+        (str) String of letters
+    """
+    return ''.join(c if c.isalnum() else '_' for c in str(skill_id))
+
+
+def get_temp_path(*args) -> str:
+    """
+    Generate a valid path in the system temp directory.
 
     This method accepts one or more strings as arguments. The arguments are
     joined and returned as a complete path inside the systems temp directory.
@@ -39,45 +84,88 @@ def get_temp_path(*args):
     return path
 
 
-def get_cache_directory(folder):
-    # optional import to use ram for cache
-    # does not work in windows!
-    try:
-        from memory_tempfile import MemoryTempfile
-    except ImportError:
-        MemoryTempfile = None
-    if os.name == 'nt' or not MemoryTempfile:
-        path = get_temp_path(folder)
-    else:
-        path = join(MemoryTempfile(fallback=True).gettempdir(), folder)
+def get_cache_directory(folder: str) -> str:
+    """
+    Get a temporary cache directory, preferably in RAM.
+    Note that only Linux use RAM.
+    @param folder: base path to use for cache
+    @return: valid cache path
+    """
+    path = get_temp_path(folder)
+    if platform == 'linux':
+        try:
+            from memory_tempfile import MemoryTempfile
+            path = join(MemoryTempfile(fallback=True).gettempdir(), folder)
+        except ImportError:
+            pass
+        except Exception as e:
+            LOG.exception(e)
     os.makedirs(path, exist_ok=True)
     return path
 
 
-def resolve_ovos_resource_file(res_name):
-    """Convert a resource into an absolute filename.
+def resolve_ovos_resource_file(res_name: str,
+                               extra_res_dirs: list = None) -> Optional[str]:
+    """
+    Convert a resource into an absolute filename.
     used internally for ovos resources
     """
     # First look for fully qualified file (e.g. a user setting)
     if os.path.isfile(res_name):
         return res_name
 
-    # now look in bundled ovos resources
+    if extra_res_dirs:
+        for res_dir in extra_res_dirs:
+            filename = join(res_dir, res_name)
+            if os.path.isfile(filename):
+                return filename
+
+    # now look in bundled ovos-utils resources
     filename = join(dirname(__file__), "res", res_name)
     if os.path.isfile(filename):
         return filename
 
-    # let's look in mycroft/ovos-core if it's installed
-    path = search_mycroft_core_location()
-    if path:
-        filename = join(path, "mycroft", "res", res_name)
+    # let's look in ovos_workshop if it's installed
+    # (default skill resources live here)
+    try:
+        import ovos_workshop
+        core_root = dirname(ovos_workshop.__file__)
+        filename = join(core_root, "res", res_name)
         if os.path.isfile(filename):
             return filename
+    except ImportError:
+        pass
+
+    # let's look in ovos_gui if it's installed
+    # (default GUI resources live here)
+    try:
+        import ovos_gui
+        core_root = dirname(ovos_gui.__file__)
+        filename = join(core_root, "res", res_name)
+        if os.path.isfile(filename):
+            return filename
+    except ImportError:
+        pass
+
+    # let's look in mycroft/ovos-core if it's installed
+    # (default core resources live here / backwards compat)
+    # TODO - remove me soon, spams deprecation logs
+    try:
+        import mycroft
+        core_root = dirname(mycroft.__file__)
+        filename = join(core_root, "res", res_name)
+        if os.path.isfile(filename):
+            return filename
+    except ImportError:
+        pass
+
     return None  # Resource cannot be resolved
 
 
-def resolve_resource_file(res_name, root_path=None, config=None):
-    """Convert a resource into an absolute filename.
+def resolve_resource_file(res_name: str, root_path: Optional[str] = None,
+                          config: dict = None) -> Optional[str]:
+    """
+    Convert a resource into an absolute filename.
 
     Resource names are in the form: 'filename.ext'
     or 'path/filename.ext'
@@ -97,13 +185,19 @@ def resolve_resource_file(res_name, root_path=None, config=None):
 
     Args:
         res_name (str): a resource path/name
+        root_path: Optional root path to check
         config (dict): mycroft.conf, to read data directory from
     Returns:
         str: path to resource or None if no resource found
     """
     if config is None:
-        from ovos_utils.configuration import read_mycroft_config
-        config = read_mycroft_config()
+        log_deprecation(f"Expected a dict config and got None.", "0.1.0")
+        try:
+            from ovos_config.config import read_mycroft_config
+            config = read_mycroft_config()
+        except ImportError:
+            LOG.warning("Config not provided and ovos_config not available")
+            config = dict()
 
     # First look for fully qualified file (e.g. a user setting)
     if os.path.isfile(res_name):
@@ -128,30 +222,32 @@ def resolve_resource_file(res_name, root_path=None, config=None):
     return None  # Resource cannot be resolved
 
 
-def read_vocab_file(path):
-    """ Read voc file.
+def read_vocab_file(path: str) -> List[List[str]]:
+    """
+    Read voc file.
 
-        This reads a .voc file, stripping out empty lines comments and expand
-        parentheses. It returns each line as a list of all expanded
-        alternatives.
+    This reads a .voc file, stripping out empty lines comments and expand
+    parentheses. It returns each line as a list of all expanded
+    alternatives.
 
-        Args:
-            path (str): path to vocab file.
+    Args:
+        path (str): path to vocab file.
 
-        Returns:
-            List of Lists of strings.
+    Returns:
+        List of Lists of strings.
     """
     vocab = []
     with open(path, 'r', encoding='utf8') as voc_file:
         for line in voc_file.readlines():
             if line.startswith('#') or line.strip() == '':
                 continue
-            vocab.append(expand_options(line.lower()))
+            vocab.append(expand_template(line.lower()))
     return vocab
 
 
-def load_regex_from_file(path, skill_id):
-    """Load regex from file
+def load_regex_from_file(path: str, skill_id: str) -> List[str]:
+    """
+    Load regex from file
     The regex is sent to the intent handler using the message bus
 
     Args:
@@ -179,8 +275,9 @@ def load_regex_from_file(path, skill_id):
     return regexes
 
 
-def load_vocabulary(basedir, skill_id):
-    """Load vocabulary from all files in the specified directory.
+def load_vocabulary(basedir: str, skill_id: str) -> dict:
+    """
+    Load vocabulary from all files in the specified directory.
 
     Args:
         basedir (str): path of directory to load from (will recurse)
@@ -188,7 +285,6 @@ def load_vocabulary(basedir, skill_id):
     Returns:
         dict with intent_type as keys and list of list of lists as value.
     """
-    from ovos_utils.intents.intent_service_interface import to_alnum
 
     vocabs = {}
     for path, _, files in walk(basedir):
@@ -201,13 +297,12 @@ def load_vocabulary(basedir, skill_id):
     return vocabs
 
 
-def load_regex(basedir, skill_id):
-    """Load regex from all files in the specified directory.
+def load_regex(basedir: str, skill_id: str) -> List[List[str]]:
+    """
+    Load regex from all files in the specified directory.
 
     Args:
         basedir (str): path of directory to load from
-        bus (messagebus emitter): messagebus instance used to send the vocab to
-                                  the intent service
         skill_id (str): skill identifier
     """
     regexes = []
@@ -218,8 +313,9 @@ def load_regex(basedir, skill_id):
     return regexes
 
 
-def read_value_file(filename, delim):
-    """Read value file.
+def read_value_file(filename: str, delim: str) -> collections.OrderedDict:
+    """
+    Read value file.
 
     The value file is a simple csv structure with a key and value.
 
@@ -246,8 +342,9 @@ def read_value_file(filename, delim):
     return result
 
 
-def read_translated_file(filename, data):
-    """Read a file inserting data.
+def read_translated_file(filename: str, data: dict) -> Optional[List[str]]:
+    """
+    Read a file inserting data.
 
     Args:
         filename (str): file to read
@@ -265,37 +362,68 @@ def read_translated_file(filename, data):
 
 
 class FileWatcher:
-    def __init__(self, files, callback, recursive=False, ignore_creation=False):
+    def __init__(self, files: List[str], callback: callable,
+                 recursive: bool = False, ignore_creation: bool = False):
+        """
+        Initialize a FileWatcher to monitor the specified files for changes
+        @param files: list of paths to monitor for file changes
+        @param callback: function to call on file change with modified file path
+        @param recursive: If true, recursively include directory contents
+        @param ignore_creation: If true, ignore file creation events
+        """
         self.observer = Observer()
         self.handlers = []
         for file_path in files:
-            watch_dir = dirname(file_path)
-            self.observer.schedule(FileEventHandler(file_path, callback, ignore_creation),
+            if os.path.isfile(file_path):
+                watch_dir = dirname(file_path)
+            else:
+                watch_dir = file_path
+            self.observer.schedule(FileEventHandler(file_path, callback,
+                                                    ignore_creation),
                                    watch_dir, recursive=recursive)
         self.observer.start()
 
     def shutdown(self):
+        """
+        Remove observer scheduled events and stop the observer.
+        """
         self.observer.unschedule_all()
         self.observer.stop()
 
 
 class FileEventHandler(FileSystemEventHandler):
-    def __init__(self, file_path, callback, ignore_creation=False):
+    def __init__(self, file_path: str, callback: callable,
+                 ignore_creation: bool = False):
+        """
+        Create a handler for file change events
+        @param file_path: file_path being watched Unused(?)
+        @param callback: function to call on file change with modified file path
+        @param ignore_creation: if True, only track file modification events
+        """
         super().__init__()
         self._callback = callback
         self._file_path = file_path
-        self._debounce = 1
-        self._last_update = 0
         if ignore_creation:
             self._events = ('modified')
         else:
             self._events = ('created', 'modified')
+        self._changed_files = []
+        self._lock = RLock()
 
     def on_any_event(self, event):
         if event.is_directory:
             return
-        elif event.event_type in self._events:
-            if event.src_path == self._file_path:
-                if time.time() - self._last_update >= self._debounce:
-                    self._callback(event.src_path)
-                    self._last_update = time.time()
+        with self._lock:
+            if event.event_type == "closed":
+                if event.src_path in self._changed_files:
+                    self._changed_files.remove(event.src_path)
+                    # fire event, it is now safe
+                    try:
+                        self._callback(event.src_path)
+                    except:
+                        LOG.exception("An error occurred handling file "
+                                      "change event callback")
+
+            elif event.event_type in self._events:
+                if event.src_path not in self._changed_files:
+                    self._changed_files.append(event.src_path)
