@@ -1,14 +1,19 @@
-import tempfile
-import time
-
 import os
 import os.path
+import tempfile
+import time
+import warnings
+from ovos_utils.log import LOG, log_deprecation, deprecated
 
-from ovos_utils.configuration import read_mycroft_config
-from ovos_utils.log import LOG
+warnings.warn(
+    "ovos_utils.signal module has been deprecated without replacement`",
+    DeprecationWarning,
+    stacklevel=2,
+)
 
 
-def get_ipc_directory(domain=None):
+@deprecated("ovos_utils.signal module has been deprecated!", "0.2.0")
+def get_ipc_directory(domain=None, config=None):
     """Get the directory used for Inter Process Communication
 
     Files in this folder can be accessed by different processes on the
@@ -17,11 +22,19 @@ def get_ipc_directory(domain=None):
     Args:
         domain (str): The IPC domain.  Basically a subdirectory to prevent
             overlapping signal filenames.
+        config (dict): mycroft.conf, to read ipc directory from
 
     Returns:
         str: a path to the IPC directory
     """
-    config = read_mycroft_config()
+    if config is None:
+        log_deprecation(f"Expected a dict config and got None.", "0.1.0")
+        try:
+            from ovos_config.config import Configuration
+            config = Configuration()
+        except ImportError:
+            LOG.warning("Config not provided and ovos_config not available")
+            config = dict()
     path = config.get("ipc_path")
     if not path:
         # If not defined, use /tmp/mycroft/ipc
@@ -29,6 +42,7 @@ def get_ipc_directory(domain=None):
     return ensure_directory_exists(path, domain)
 
 
+@deprecated("use 'from ovos_utils.file_utils import ensure_directory_exists' instead", "0.2.0")
 def ensure_directory_exists(directory, domain=None):
     """ Create a directory and give access rights to all
 
@@ -39,26 +53,11 @@ def ensure_directory_exists(directory, domain=None):
     Returns:
         str: a path to the directory
     """
-    if domain:
-        directory = os.path.join(directory, domain)
-
-    # Expand and normalize the path
-    directory = os.path.normpath(directory)
-    directory = os.path.expanduser(directory)
-
-    if not os.path.isdir(directory):
-        try:
-            save = os.umask(0)
-            os.makedirs(directory, 0o777)  # give everyone rights to r/w here
-        except OSError:
-            LOG.warning("Failed to create: " + directory)
-            pass
-        finally:
-            os.umask(save)
-
-    return directory
+    from ovos_utils.file_utils import ensure_directory_exists as _ede
+    return _ede(directory, domain)
 
 
+@deprecated("ovos_utils.signal module has been deprecated!", "0.2.0")
 def create_file(filename):
     """ Create the file filename and create any directories needed
 
@@ -73,22 +72,26 @@ def create_file(filename):
         f.write('')
 
 
-def create_signal(signal_name):
+@deprecated("ovos_utils.signal module has been deprecated!", "0.2.0")
+def create_signal(signal_name, config=None):
     """Create a named signal
 
     Args:
         signal_name (str): The signal's name.  Must only contain characters
             valid in filenames.
+        config (dict): mycroft.conf, to read ipc directory from
     """
     try:
-        path = os.path.join(get_ipc_directory(), "signal", signal_name)
+        path = os.path.join(get_ipc_directory(config=config),
+                            "signal", signal_name)
         create_file(path)
         return os.path.isfile(path)
     except IOError:
         return False
 
 
-def check_for_signal(signal_name, sec_lifetime=0):
+@deprecated("ovos_utils.signal module has been deprecated!", "0.2.0")
+def check_for_signal(signal_name, sec_lifetime=0, config=None):
     """See if a named signal exists
 
     Args:
@@ -97,11 +100,13 @@ def check_for_signal(signal_name, sec_lifetime=0):
         sec_lifetime (int, optional): How many seconds the signal should
             remain valid.  If 0 or not specified, it is a single-use signal.
             If -1, it never expires.
+        config (dict): mycroft.conf, to read ipc directory from
 
     Returns:
         bool: True if the signal is defined, False otherwise
     """
-    path = os.path.join(get_ipc_directory(), "signal", signal_name)
+    path = os.path.join(get_ipc_directory(config=config),
+                        "signal", signal_name)
     if os.path.isfile(path):
         if sec_lifetime == 0:
             # consume this single-use signal
